<?php

declare(strict_types=1);

namespace mirzaev\arming_bot\controllers;

// Files of the project
use mirzaev\arming_bot\controllers\core,
	mirzaev\arming_bot\models\session,
	mirzaev\arming_bot\models\account as model;

// Framework for ArangoDB
use mirzaev\arangodb\document;

/**
 * Controller of account
 *
 * @package mirzaev\arming_bot\controllers
 *
 * @license http://www.wtfpl.net/ Do What The Fuck You Want To Public License
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class account extends core
{
	/**
	 * Registry of errors
	 */
	protected array $errors = [
		'session' => [],
		'account' => []
	];

	/**
	 * Write to the buffer
	 *
	 * @param array $parameters Parameters of the request (POST + GET)
	 *
	 * @return void
	 */
	public function write(array $parameters = []): void
	{
		if (!empty($parameters) && $this->account instanceof model) {
			// Received parameters and initialized account

			// Declaring the buffer of deserialized parameters
			$deserialized = [];

			foreach ($parameters as $name => $value) {
				// Iterate over parameters

				// Validation of the parameter value
				if (mb_strlen($value) > 4096) continue;

				// Declaring the buffer of deserialized parameter
				$parameter = null;

				// Deserializing name to multidimensional array
				foreach (array_reverse(explode('_', $name)) as $key)
					$parameter = [$key => $parameter ?? (json_validate($value) ? json_decode($value, true, 10) : $value)];

				// Writing into the buffer of deserialized parameters
				$deserialized = array_merge_recursive($parameter, $deserialized);
			}


			// Write to the account document from ArangoDB
			if (!empty($deserialized)) $this->account->write($deserialized, $this->errors['account']);
		}
	}
}
