<?php

declare(strict_types=1);

namespace mirzaev\arming_bot\controllers;

// Files of the project
use mirzaev\arming_bot\controllers\core,
	mirzaev\arming_bot\models\cart as model,
	mirzaev\arming_bot\models\product,
	mirzaev\arming_bot\models\menu,
	mirzaev\arming_bot\models\enumerations\language;

// Framework for ArangoDB
use mirzaev\arangodb\document;

/**
 * Controller of cart
 *
 * @package mirzaev\arming_bot\controllers
 *
 * @license http://www.wtfpl.net/ Do What The Fuck You Want To Public License
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class cart extends core
{
	/**
	 * Instance of the cart
	 */
	protected readonly ?model $cart;

	/**
	 * Registry of errors
	 */
	protected array $errors = [
		'session' => [],
		'account' => [],
		'cart' => [],
		'menu' => [],
	];

	/**
	 * Cart
	 *
	 * @param array $parameters Parameters of the request (POST + GET)
	 */
	public function index(array $parameters = []): ?string
	{
		if (isset($menu)) {
			// 

		} else {
			// Not received ... menu

			// Search for filters and write to the buffer of global variables of view templater
			$this->view->menu = menu::_read(
				return: 'MERGE(d, { name: d.name.@language })',
				sort: 'd.style.order ASC, d.created DESC, d._key DESC',
				amount: 4,
				parameters: ['language' => $this->language->name],
				errors: $this->errors['menu']
			);
		}

		// Initializing the cart
		$this->cart = $this->account?->cart() ?? $this->session?->cart();

		// Initializing the cart data
		$this->view->cart = [
			'summary' => $this->cart?->summary(currency: $this->currency),
			'products' => $this->cart?->products(language: $this->language, currency: $this->currency)
		];

		// Initializing types of avaiabld deliveries
		$this->view->deliveries = [
			'cdek' => [
				'label' => 'CDEK'
			]
		];

		if ($_SERVER['REQUEST_METHOD'] === 'GET') {
			// GET request

			// Exit (success)
			return $this->view->render('cart/page.html', [
				'h2' => $this->language === language::ru ? 'Корзина' : 'Cart' // @see https://git.mirzaev.sexy/mirzaev/huesos/issues/1
			]);
		} else if ($_SERVER['REQUEST_METHOD'] === 'POST') {
			// POST request

			// Initializing a response headers
			header('Content-Type: application/json');
			header('Content-Encoding: none');
			header('X-Accel-Buffering: no');

			// Initializing of the output buffer
			ob_start();

			// Generating the reponse
			echo json_encode(
				[
					'main' => '',
					'errors' => $this->errors
				]
			);

			// Initializing a response headers
			header('Content-Length: ' . ob_get_length());

			// Sending and deinitializing of the output buffer
			ob_end_flush();
			flush();

			// Exit (success)
			return null;
		}

		// Exit (fail)
		return null;
	}

	/**
	 * Product
	 *
	 * Write or delete the product in the cart
	 *
	 * @param array $parameters Parameters of the request (POST + GET)
	 *
	 * @todo
	 * 1. Add a limit on adding products to the cart based on the number of products in stock
	 */
	public function product(array $parameters = []): ?string
	{
		if ($_SERVER['REQUEST_METHOD'] === 'POST') {
			// POST request

			// Declaring of the buffer with amount of the product in the cart
			$amount = 0;

			// Validating @todo add throwing errors
			$identifier = null;
			if (isset($parameters['identifier']) && preg_match('/[\d]+/', urldecode($parameters['identifier']), $matches)) $identifier = (int) $matches[0];

			if (isset($identifier)) {
				// Received and validated identfier of the product

				// Search for the product
				$product = product::read(
					filter: "d.identifier == @identifier && d.deleted != true && d.hidden != true",
					sort: 'd.created DESC',
					amount: 1,
					parameters: ['identifier' => $identifier],
					errors: $this->errors['cart']
				);

				if ($product instanceof product) {
					// Initialized the product

					// Initializing the cart
					$this->cart = $this->account?->cart() ?? $this->session?->cart();

					if ($this->cart instanceof model) {
						// Initialized the cart

						// Initializing the buffer with amount of the product in the cart 
						$amount = $this->cart->count(product: $product, limit: 100, errors: $this->errors['cart']) ?? 0;

						if ($this->cart instanceof model) {
							// Initialized the cart

							// Validating @todo add throwing errors
							$type = null;
							if (isset($parameters['type']) && preg_match('/[\w]+/', urldecode($parameters['type']), $matches)) $type = $matches[0];

							if (isset($type)) {
								// Received and validated type of action with the product

								if ($type === 'toggle') {
									// Write the product to the cart if is not in the cart and vice versa

									if ($amount > 0) {
										// The cart has the product

										// Deleting the product from the cart
										$this->cart->delete(product: $product, amount: $amount, errors: $this->errors['cart']);

										// Reinitializing the buffer with amount of the product in the cart
										$amount = 0;
									} else {
										// The cart has no the product

										// Writing the product to the cart
										$this->cart->write(product: $product, amount: 1, errors: $this->errors['cart']);

										// Reinitializing the buffer with amount of the product in the cart
										$amount = 1;
									}
								} else {
									// Received not the "toggle" command

									// Validating @todo add throwing errors
									$_amount = null;
									if (isset($parameters['amount']) && preg_match('/[\d]+/', urldecode($parameters['amount']), $matches)) $_amount = (int) $matches[0];

									if (isset($_amount)) {
										// Received and validated amount parameter for action with the product

										if ($type === 'write') {
											// Increase amount of the product in the cart

											if ($amount + $_amount < 101) {
												// Validated amount to wrting

												// Writing the product to the cart
												$this->cart->write(product: $product, amount: $_amount, errors: $this->errors['cart']);

												// Reinitialize the buffer with amount of the product in the cart
												$amount += $_amount;
											}
										} else if ($type === 'delete') {
											// Decrease amount of the product in the cart

											if ($amount - $_amount > -1) {
												// Validated amount to deleting

												// Deleting the product from the cart
												$this->cart->delete(product: $product, amount: $_amount, errors: $this->errors['cart']);

												// Reinitialize the buffer with amount of the product in the cart
												$amount -= $_amount;
											}
										} else if ($type === 'set') {
											// Set amount of the product in the cart

											if ($_amount > -1 && $_amount < 101) {
												// Validated amount to setting

												if ($_amount > $amount) {
													// Requested amount more than actual amount of the product in the cart

													// Writing the product from the cart
													$this->cart->write(product: $product, amount: $_amount - $amount, errors: $this->errors['cart']);
												} else {
													// Requested amount less than actual amount of the product in the cart

													// Deleting the product from the cart
													$this->cart->delete(product: $product, amount: $amount - $_amount, errors: $this->errors['cart']);
												}

												// Reinitializing the buffer with amount of the product in the cart
												$amount = $_amount;
											}
										}
									}
								}
							}
						}
					}
				}
			}

			// Initializing a response headers
			header('Content-Type: application/json');
			header('Content-Encoding: none');
			header('X-Accel-Buffering: no');

			// Initializing of the output buffer
			ob_start();

			// Generating the reponse
			echo json_encode(
				[
					'amount' => $amount, // $amount does not store a real value, but is calculated without a repeated request to ArangoDB
					'errors' => $this->errors
				]
			);

			// Initializing a response headers
			header('Content-Length: ' . ob_get_length());

			// Sending and deinitializing of the output buffer
			ob_end_flush();
			flush();

			// Exit (success)
			return null;
		}

		// Exit (fail)
		return null;
	}

	/**
	 * Summary
	 *
	 * @param array $parameters Parameters of the request (POST + GET)
	 */
	public function summary(array $parameters = []): ?string
	{
		if ($_SERVER['REQUEST_METHOD'] === 'POST') {
			// POST request

			// Initializing the cart
			$this->cart = $this->account?->cart() ?? $this->session?->cart();

			if ($this->cart instanceof model) {
				// Initialized the cart

				// Initializing summary data of the cart
				$summary = $this->cart?->summary(currency: $this->currency, errors: $this->errors['cart']);

				// Initializing response headers
				header('Content-Type: application/json');
				header('Content-Encoding: none');
				header('X-Accel-Buffering: no');

				// Initializing of the output buffer
				ob_start();

				// Generating the reponse
				echo json_encode(
					[
						'cost' => $summary['cost'] ?? 0,
						'amount' => $summary['amount'] ?? 0,
						'errors' => $this->errors
					]
				);

				// Initializing a response headers
				header('Content-Length: ' . ob_get_length());

				// Sending and deinitializing of the output buffer
				ob_end_flush();
				flush();

				// Exit (success)
				return null;
			}
		}

		// Exit (fail)
		return null;
	}
}
