<?php

declare(strict_types=1);

namespace mirzaev\arming_bot\models\deliveries;

// Files of the project
use mirzaev\arming_bot\models\core,
	mirzaev\arming_bot\models\traits\document as document_trait,
	mirzaev\arming_bot\models\interfaces\document as document_interface,
	mirzaev\arming_bot\models\interfaces\collection as collection_interface;

// The HTTP PSR-18 adapter for Guzzle HTTP client
use Http\Adapter\Guzzle7\Client as guzzle;

// Framework for CDEK
use CdekSDK2\Client as client,
	CdekSDK2\Dto\CityList as cities;

// Built-in libraries
use exception;

/**
 * Model of CDEK
 *
 * @package mirzaev\arming_bot\models\deliveries
 *
 * @license http://www.wtfpl.net/ Do What The Fuck You Want To Public License
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class cdek extends core implements document_interface, collection_interface
{
	use document_trait;

	/**
	 * Name of the collection in ArangoDB
	 */
	final public const string COLLECTION = 'delivery';

	/**
	 * Calculate
	 *
	 * Calculate delivery by CDEK
	 *
	 * @param array &$errors Registry of errors
	 *
	 * @return 
	 */
	public static function calculate(array &$errors = []): static|null
	{
		try {
			//
			/* $client = new client(new guzzle, 'account', 'secure'); */
			$client = new client(new guzzle);
			$client->setTest(true);

			$result = $client->cities()->getFiltered(['country_codes' => 'RU', 'city' => 'зеленогорск']);

			if ($result->isOk()) {
				//

				//Запрос успешно выполнился
				$cities = $client->formatResponseList($result, cities::class);

				foreach ($cities->items as $city) {
					var_dump($city);
				}

				die;
			}
		} catch (exception $e) {
			// Writing to the registry of errors
			$errors[] = [
				'text' => $e->getMessage(),
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'stack' => $e->getTrace()
			];
		}

		// Exit (fail)
		return null;
	}
}
