"use strict";

/**
 * @name Delivery
 *
 * @description
 * Implements actions with delivery
 *
 * @license http://www.wtfpl.net/ Do What The Fuck You Want To Public License
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
export default class delivery {
	/**
	 * @name Type of the program
	 */
	static type = "module";

	/**
	 * @name Company
	 *
	 * @description
	 * Choose a delivery company (interface)
	 *
	 * @param {HTNLElement} element Handler element
	 * @param {string} identifier Identifier of the delivery company
	 * @param {bool} force Ignore the damper? (false)
	 *
	 * @return {bool} Execution completed with an error?
	 */
	static company = (element, identifier, force = false) => {
		core.modules.connect("damper").then(
			() => {
				// Imported the damper module

				// Execute under damper
				this.company.damper(element, identifier, force);
			},
			() => {
				// Not imported the damper module

				// Execute
				this.company.system(element, identifier, force);
			},
		);

		// Exit (success)
		return false;
	};

	/**
	 * @name City
	 *
	 * @description
	 * Write name of a city for delivery (interface)
	 *
	 * @param {HTNLInputElement} element Handler element <input>
	 * @param {bool} force Ignore the damper? (false)
	 *
	 * @return {bool} Execution completed with an error?
	 */
	static company = (element, force = false) => {
		core.modules.connect("damper").then(
			() => {
				// Imported the damper module

				// Execute under damper
				this.city.damper(element, identifier, force);
			},
			() => {
				// Not imported the damper module

				// Execute
				this.company.system(element, identifier, force);
			},
		);

		// Exit (success)
		return false;
	};
}

core.modules.connect("damper").then(() => {
	// Imported the damper module

	Object.assign(
		delivery.company,
		{
			/**
			 * @name Write
			 *
			 * @description
			 * Choose a delivery company (damper)
			 *
			 * @param {HTNLElement} element Handler element
			 * @param {string} identifier Identifier of the delivery company
			 * @param {bool} force Ignore the damper? (false)
			 *
			 * @return {Promise}
			 */
			damper: core.damper(
				(...variables) => delivery.company.system(...variables),
				300,
				3,
			),
		},
	);
});

Object.assign(
	delivery.company,
	{
		/**
		 * @name Write
		 *
		 * @description
		 * Choose a delivery company (system)
		 *
		 * @param {string} identifier Identifier of the delivery company
		 *
		 * @return {Promise}
		 */
		async system(
			type,
			value,
			resolve = () => {},
			reject = () => {},
		) {
			try {
				if (
					typeof type === "string" &&
					(typeof value === "string" || typeof valye === "number")
				) {
					// Received and validated required arguments

					// Sending request to the server
					return await core.buffer.write(`delivery_${type}`, value)
						.then(
							(json) => {
								try {
									

									// Exit (success)
									resolve(json);
								} catch (e) {
									// Exit (fail)
									reject(e);
								}
							},
							() => reject(),
						);
				}
			} catch (e) {
				// Exit (fail)
				reject(e);
			}
		},
	},
);

// Connecting to the core
if (!core.delivery) core.delivery = delivery;
